@@ -24,13 +24,11 @@
 #include "gnome-wp-info.h"
 #include "gnome-wp-item.h"
 #include "gnome-wp-xml.h"
-#include "wp-cellrenderer.h"
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <string.h>
 #include <gconf/gconf-client.h>
-#include <libgnomeui/gnome-thumbnail.h>
-#define GNOME_DESKTOP_USE_UNSTABLE_API
+#include <libgnomeui/gnome-desktop-thumbnail.h>
 #include <libgnomeui/gnome-bg.h>
 
 enum {
@@ -84,7 +82,6 @@ get_selected_item (AppearanceData *data,
   if (selected != NULL)
   {
     GtkTreeIter sel_iter;
-    gchar *wpfile;
 
     gtk_tree_model_get_iter (data->wp_model, &sel_iter,
                              selected->data);
@@ -95,10 +92,7 @@ get_selected_item (AppearanceData *data,
     if (iter)
       *iter = sel_iter;
 
-    gtk_tree_model_get (data->wp_model, &sel_iter, 2, &wpfile, -1);
-
-    item = g_hash_table_lookup (data->wp_hash, wpfile);
-    g_free (wpfile);
+    gtk_tree_model_get (data->wp_model, &sel_iter, 1, &item, -1);
   }
 
   return item;
@@ -131,11 +125,12 @@ static void on_item_changed (GnomeBG *bg, AppearanceData *data) {
 
     g_signal_handlers_block_by_func (bg, G_CALLBACK (on_item_changed), data);
 
-    pixbuf = gnome_wp_item_get_thumbnail (item, data->thumb_factory);
+    pixbuf = gnome_wp_item_get_thumbnail (item,
+                                          data->thumb_factory,
+                                          data->thumb_width,
+                                          data->thumb_height);
     if (pixbuf) {
-      gtk_list_store_set (GTK_LIST_STORE (data->wp_model), &iter,
-                          0, pixbuf,
-                         -1);
+      gtk_list_store_set (GTK_LIST_STORE (data->wp_model), &iter, 0, pixbuf, -1);
       g_object_unref (pixbuf);
     }
 
@@ -157,13 +152,14 @@ wp_props_load_wallpaper (gchar *key,
 
   gtk_list_store_append (GTK_LIST_STORE (data->wp_model), &iter);
 
-  pixbuf = gnome_wp_item_get_thumbnail (item, data->thumb_factory);
+  pixbuf = gnome_wp_item_get_thumbnail (item, data->thumb_factory,
+                                        data->thumb_width,
+                                        data->thumb_height);
   gnome_wp_item_update_description (item);
 
   gtk_list_store_set (GTK_LIST_STORE (data->wp_model), &iter,
                       0, pixbuf,
-                      1, item->description,
-                      2, item->filename,
+                      1, item,
                       -1);
 
   if (pixbuf != NULL)
@@ -183,7 +179,7 @@ wp_add_image (AppearanceData *data,
 
   if (!filename)
     return NULL;
-  
+
   item = g_hash_table_lookup (data->wp_hash, filename);
 
   if (item != NULL)
@@ -212,10 +208,12 @@ wp_add_images (AppearanceData *data,
                GSList *images)
 {
   GdkWindow *window;
+  GtkWidget *w;
   GdkCursor *cursor;
   GnomeWPItem *item;
 
-  window = glade_xml_get_widget (data->xml, "appearance_window")->window;
+  w = appearance_capplet_get_widget (data, "appearance_window");
+  window = gtk_widget_get_window (w);
 
   item = NULL;
   cursor = gdk_cursor_new_for_display (gdk_display_get_default (),
@@ -315,11 +313,10 @@ wp_scale_type_changed (GtkComboBox *combobox,
 
   item->options = gtk_combo_box_get_active (GTK_COMBO_BOX (data->wp_style_menu));
 
-  pixbuf = gnome_wp_item_get_thumbnail (item, data->thumb_factory);
-  gtk_list_store_set (GTK_LIST_STORE (data->wp_model),
-                      &iter,
-                      0, pixbuf,
-                      -1);
+  pixbuf = gnome_wp_item_get_thumbnail (item, data->thumb_factory,
+                                        data->thumb_width,
+                                        data->thumb_height);
+  gtk_list_store_set (GTK_LIST_STORE (data->wp_model), &iter, 0, pixbuf, -1);
   if (pixbuf != NULL)
     g_object_unref (pixbuf);
 
@@ -343,10 +340,10 @@ wp_shade_type_changed (GtkWidget *combobox,
 
   item->shade_type = gtk_combo_box_get_active (GTK_COMBO_BOX (data->wp_color_menu));
 
-  pixbuf = gnome_wp_item_get_thumbnail (item, data->thumb_factory);
-  gtk_list_store_set (GTK_LIST_STORE (data->wp_model), &iter,
-                      0, pixbuf,
-                      -1);
+  pixbuf = gnome_wp_item_get_thumbnail (item, data->thumb_factory,
+                                        data->thumb_width,
+                                        data->thumb_height);
+  gtk_list_store_set (GTK_LIST_STORE (data->wp_model), &iter, 0, pixbuf, -1);
   if (pixbuf != NULL)
     g_object_unref (pixbuf);
 
@@ -411,6 +408,7 @@ wp_remove_wallpaper (GtkWidget *widget,
       path = gtk_tree_path_new_first ();
 
     gtk_icon_view_select_path (data->wp_view, path);
+    gtk_icon_view_set_cursor (data->wp_view, path, NULL, FALSE);
     gtk_tree_path_free (path);
   }
 }
@@ -626,10 +624,9 @@ wp_create_filechooser (AppearanceData *data)
   GtkFileFilter *filter;
 
   data->wp_filesel = GTK_FILE_CHOOSER (
-  		     gtk_file_chooser_dialog_new_with_backend (_("Add Wallpaper"),
-                     GTK_WINDOW (glade_xml_get_widget (data->xml, "appearance_window")),
+                     gtk_file_chooser_dialog_new (_("Add Wallpaper"),
+                     GTK_WINDOW (appearance_capplet_get_widget (data, "appearance_window")),
                      GTK_FILE_CHOOSER_ACTION_OPEN,
-                     "gtk+",
                      GTK_STOCK_CANCEL,
                      GTK_RESPONSE_CANCEL,
                      GTK_STOCK_OPEN,
@@ -710,14 +707,16 @@ wp_drag_received (GtkWidget *widget,
     GSList *realuris = NULL;
     gchar **uris;
 
-    uris = g_uri_list_extract_uris ((gchar *) selection_data->data);
+    uris = g_uri_list_extract_uris ((gchar *) gtk_selection_data_get_data (selection_data));
     if (uris != NULL)
     {
+      GtkWidget *w;
       GdkWindow *window;
       GdkCursor *cursor;
       gchar **uri;
 
-      window = glade_xml_get_widget (data->xml, "appearance_window")->window;
+      w = appearance_capplet_get_widget (data, "appearance_window");
+      window = gtk_widget_get_window (w);
 
       cursor = gdk_cursor_new_for_display (gdk_display_get_default (),
              GDK_WATCH);
@@ -756,7 +755,7 @@ wp_drag_get_data (GtkWidget *widget,
 
       uris[0] = g_filename_to_uri (item->filename, NULL, NULL);
       uris[1] = NULL;
-    
+
       gtk_selection_data_set_uris (selection_data, uris);
 
       g_free (uris[0]);
@@ -773,7 +772,6 @@ wp_view_tooltip_cb (GtkWidget  *widget,
                     AppearanceData *data)
 {
   GtkTreeIter iter;
-  gchar *wpfile;
   GnomeWPItem *item;
 
   if (gtk_icon_view_get_tooltip_context (data->wp_view,
@@ -783,10 +781,7 @@ wp_view_tooltip_cb (GtkWidget  *widget,
                                          NULL,
                                          &iter))
     {
-      gtk_tree_model_get (data->wp_model, &iter, 2, &wpfile, -1);
-      item = g_hash_table_lookup (data->wp_hash, wpfile);
-      g_free (wpfile);
-
+      gtk_tree_model_get (data->wp_model, &iter, 1, &item, -1);
       gtk_tooltip_set_markup (tooltip, item->description);
 
       return TRUE;
@@ -800,31 +795,25 @@ wp_list_sort (GtkTreeModel *model,
               GtkTreeIter *a, GtkTreeIter *b,
               AppearanceData *data)
 {
-  gchar *foo, *bar;
-  gchar *desca, *descb;
+  GnomeWPItem *itema, *itemb;
   gint retval;
 
-  gtk_tree_model_get (model, a, 1, &desca, 2, &foo, -1);
-  gtk_tree_model_get (model, b, 1, &descb, 2, &bar, -1);
+  gtk_tree_model_get (model, a, 1, &itema, -1);
+  gtk_tree_model_get (model, b, 1, &itemb, -1);
 
-  if (!strcmp (foo, "(none)"))
+  if (!strcmp (itema->filename, "(none)"))
   {
     retval =  -1;
   }
-  else if (!strcmp (bar, "(none)"))
+  else if (!strcmp (itemb->filename, "(none)"))
   {
     retval =  1;
   }
   else
   {
-    retval = g_utf8_collate (desca, descb);
+    retval = g_utf8_collate (itema->description, itemb->description);
   }
 
-  g_free (desca);
-  g_free (descb);
-  g_free (foo);
-  g_free (bar);
-
   return retval;
 }
 
@@ -858,9 +847,9 @@ wp_update_preview (GtkFileChooser *chooser,
 
     if (mime_type)
     {
-      pixbuf = gnome_thumbnail_factory_generate_thumbnail (data->thumb_factory,
-                                                           uri,
-                                                           mime_type);
+      pixbuf = gnome_desktop_thumbnail_factory_generate_thumbnail (data->thumb_factory,
+								   uri,
+								   mime_type);
     }
 
     if (pixbuf != NULL)
@@ -879,6 +868,68 @@ wp_update_preview (GtkFileChooser *chooser,
   gtk_file_chooser_set_preview_widget_active (chooser, TRUE);
 }
 
+static gboolean
+reload_item (GtkTreeModel *model,
+             GtkTreePath *path,
+             GtkTreeIter *iter,
+             AppearanceData *data)
+{
+  GnomeWPItem *item;
+  GdkPixbuf *pixbuf;
+
+  gtk_tree_model_get (model, iter, 1, &item, -1);
+
+  pixbuf = gnome_wp_item_get_thumbnail (item,
+                                        data->thumb_factory,
+                                        data->thumb_width,
+                                        data->thumb_height);
+  if (pixbuf) {
+    gtk_list_store_set (GTK_LIST_STORE (data->wp_model), iter, 0, pixbuf, -1);
+    g_object_unref (pixbuf);
+  }
+
+  return FALSE;
+}
+
+static gdouble
+get_monitor_aspect_ratio_for_widget (GtkWidget *widget)
+{
+  gdouble aspect;
+  gint monitor;
+  GdkRectangle rect;
+
+  monitor = gdk_screen_get_monitor_at_window (gtk_widget_get_screen (widget), gtk_widget_get_window (widget));
+  gdk_screen_get_monitor_geometry (gtk_widget_get_screen (widget), monitor, &rect);
+  aspect = rect.height / (gdouble)rect.width;
+
+  return aspect;
+}
+
+#define LIST_IMAGE_SIZE 108
+
+static void
+compute_thumbnail_sizes (AppearanceData *data)
+{
+  gdouble aspect;
+
+  aspect = get_monitor_aspect_ratio_for_widget (GTK_WIDGET (data->wp_view));
+  if (aspect > 1) {
+    /* portrait */
+    data->thumb_width = LIST_IMAGE_SIZE / aspect;
+    data->thumb_height = LIST_IMAGE_SIZE;
+  } else {
+    data->thumb_width = LIST_IMAGE_SIZE;
+    data->thumb_height = LIST_IMAGE_SIZE * aspect;
+  }
+}
+
+static void
+reload_wallpapers (AppearanceData *data)
+{
+  compute_thumbnail_sizes (data);
+  gtk_tree_model_foreach (data->wp_model, (GtkTreeModelForeachFunc)reload_item, data);
+}
+
 static gboolean
 wp_load_stuffs (void *user_data)
 {
@@ -888,6 +939,8 @@ wp_load_stuffs (void *user_data)
 
   data = (AppearanceData *) user_data;
 
+  compute_thumbnail_sizes (data);
+
   gnome_wp_xml_load_list (data);
   g_hash_table_foreach (data->wp_hash, (GHFunc) wp_props_load_wallpaper,
                         data);
@@ -907,7 +960,7 @@ wp_load_stuffs (void *user_data)
     g_free (uri);
     uri = NULL;
   }
-  
+
   if (uri == NULL)
     uri = g_strdup ("(none)");
 
@@ -981,22 +1034,196 @@ static void
 wp_select_after_realize (GtkWidget *widget,
                          AppearanceData *data)
 {
-  GnomeWPItem *item = get_selected_item (data, NULL);
+  GnomeWPItem *item;
 
+  g_idle_add (wp_load_stuffs, data);
+
+  item = get_selected_item (data, NULL);
   if (item == NULL)
     item = g_hash_table_lookup (data->wp_hash, "(none)");
 
   select_item (data, item, TRUE);
 }
 
+static GdkPixbuf *buttons[3];
+
+static void
+create_button_images (AppearanceData  *data)
+{
+  GtkWidget *widget = (GtkWidget*)data->wp_view;
+  GtkStyle *style = gtk_widget_get_style (widget);
+  GtkIconSet *icon_set;
+  GdkPixbuf *pixbuf, *pb, *pb2;
+  gint i, w, h;
+
+  icon_set = gtk_style_lookup_icon_set (style, "gtk-media-play");
+  pb = gtk_icon_set_render_icon (icon_set,
+                                 style,
+                                 GTK_TEXT_DIR_RTL,
+                                 GTK_STATE_NORMAL,
+                                 GTK_ICON_SIZE_MENU,
+                                 widget,
+                                 NULL);
+  pb2 = gtk_icon_set_render_icon (icon_set,
+                                  style,
+                                  GTK_TEXT_DIR_LTR,
+                                  GTK_STATE_NORMAL,
+                                  GTK_ICON_SIZE_MENU,
+                                  widget,
+                                  NULL);
+  w = gdk_pixbuf_get_width (pb);
+  h = gdk_pixbuf_get_height (pb);
+
+  for (i = 0; i < 3; i++) {
+    pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, 2 * w, h);
+    gdk_pixbuf_fill (pixbuf, 0);
+    if (i > 0)
+      gdk_pixbuf_composite (pb, pixbuf, 0, 0, w, h, 0, 0, 1, 1, GDK_INTERP_NEAREST, 255);
+    if (i < 2)
+      gdk_pixbuf_composite (pb2, pixbuf, w, 0, w, h, w, 0, 1, 1, GDK_INTERP_NEAREST, 255);
+
+    buttons[i] = pixbuf;
+  }
+
+  g_object_unref (pb);
+  g_object_unref (pb2);
+}
+
+static void
+next_frame (AppearanceData  *data,
+            GtkCellRenderer *cr,
+            gint             direction)
+{
+  GnomeWPItem *item;
+  GtkTreeIter iter;
+  GdkPixbuf *pixbuf, *pb;
+  gint frame;
+
+  pixbuf = NULL;
+
+  frame = data->frame + direction;
+  item = get_selected_item (data, &iter);
+
+  if (frame >= 0)
+    pixbuf = gnome_wp_item_get_frame_thumbnail (item,
+                                                data->thumb_factory,
+                                                data->thumb_width,
+                                                data->thumb_height,
+                                                frame);
+  if (pixbuf) {
+    gtk_list_store_set (GTK_LIST_STORE (data->wp_model), &iter, 0, pixbuf, -1);
+    g_object_unref (pixbuf);
+    data->frame = frame;
+  }
+
+  pb = buttons[1];
+  if (direction < 0) {
+    if (frame == 0)
+      pb = buttons[0];
+  }
+  else {
+    pixbuf = gnome_wp_item_get_frame_thumbnail (item,
+                                                data->thumb_factory,
+                                                data->thumb_width,
+                                                data->thumb_height,
+                                                frame + 1);
+    if (pixbuf)
+      g_object_unref (pixbuf);
+    else
+      pb = buttons[2];
+  }
+  g_object_set (cr, "pixbuf", pb, NULL);
+}
+
+static gboolean
+wp_button_press_cb (GtkWidget      *widget,
+                    GdkEventButton *event,
+                    AppearanceData *data)
+{
+  GtkCellRenderer *cell;
+  GdkEventButton *button_event = (GdkEventButton *) event;
+
+  if (event->type != GDK_BUTTON_PRESS)
+    return FALSE;
+
+  if (gtk_icon_view_get_item_at_pos (GTK_ICON_VIEW (widget),
+                                     button_event->x, button_event->y,
+                                     NULL, &cell)) {
+    if (g_object_get_data (G_OBJECT (cell), "buttons")) {
+      gint w, h;
+      GtkCellRenderer *cell2 = NULL;
+      gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
+      if (gtk_icon_view_get_item_at_pos (GTK_ICON_VIEW (widget),
+                                         button_event->x + w, button_event->y,
+                                         NULL, &cell2) && cell == cell2)
+        next_frame (data, cell, -1);
+      else
+        next_frame (data, cell, 1);
+      return TRUE;
+    }
+  }
+
+  return FALSE;
+}
+
+static void
+wp_selected_changed_cb (GtkIconView    *view,
+                        AppearanceData *data)
+{
+  GtkCellRenderer *cr;
+  GList *cells, *l;
+
+  data->frame = -1;
+
+  cells = gtk_cell_layout_get_cells (GTK_CELL_LAYOUT (data->wp_view));
+  for (l = cells; l; l = l->next) {
+    cr = l->data;
+    if (g_object_get_data (G_OBJECT (cr), "buttons"))
+      g_object_set (cr, "pixbuf", buttons[0], NULL);
+  }
+  g_list_free (cells);
+}
+
+static void
+buttons_cell_data_func (GtkCellLayout   *layout,
+                        GtkCellRenderer *cell,
+                        GtkTreeModel    *model,
+                        GtkTreeIter     *iter,
+                        gpointer         user_data)
+{
+  AppearanceData *data = user_data;
+  GtkTreePath *path;
+  GnomeWPItem *item;
+  gboolean visible;
+
+  path = gtk_tree_model_get_path (model, iter);
+
+  if (gtk_icon_view_path_is_selected (GTK_ICON_VIEW (layout), path)) {
+    item = get_selected_item (data, NULL);
+    visible = gnome_bg_changes_with_time (item->bg);
+  }
+  else
+    visible = FALSE;
+
+  g_object_set (G_OBJECT (cell), "visible", visible, NULL);
+
+  gtk_tree_path_free (path);
+}
+
+static void
+screen_monitors_changed (GdkScreen *screen,
+                         AppearanceData *data)
+{
+  reload_wallpapers (data);
+}
+
 void
 desktop_init (AppearanceData *data,
 	      const gchar **uris)
 {
-  GtkWidget *add_button;
+  GtkWidget *add_button, *w;
   GtkCellRenderer *cr;
-
-  g_object_set (gtk_settings_get_default (), "gtk-tooltip-timeout", 500, NULL);
+  char *url;
 
   data->wp_update_gconf = TRUE;
 
@@ -1008,6 +1235,16 @@ desktop_init (AppearanceData *data,
     }
   }
 
+  w = appearance_capplet_get_widget (data, "more_backgrounds_linkbutton");
+  url = gconf_client_get_string (data->client, MORE_BACKGROUNDS_URL_KEY, NULL);
+  if (url != NULL && url[0] != '\0') {
+    gtk_link_button_set_uri (GTK_LINK_BUTTON (w), url);
+    gtk_widget_show (w);
+  } else {
+    gtk_widget_hide (w);
+  }
+  g_free (url);
+
   data->wp_hash = g_hash_table_new (g_str_hash, g_str_equal);
 
   gconf_client_add_dir (data->client, WP_PATH_KEY,
@@ -1034,11 +1271,10 @@ desktop_init (AppearanceData *data,
                            (GConfClientNotifyFunc) wp_color2_changed,
                            data, NULL, NULL);
 
-  data->wp_model = GTK_TREE_MODEL (gtk_list_store_new (3, GDK_TYPE_PIXBUF,
-                                                 G_TYPE_STRING,
-                                                 G_TYPE_STRING));
+  data->wp_model = GTK_TREE_MODEL (gtk_list_store_new (2, GDK_TYPE_PIXBUF,
+                                                       G_TYPE_POINTER));
 
-  data->wp_view = GTK_ICON_VIEW (glade_xml_get_widget (data->xml, "wp_view"));
+  data->wp_view = GTK_ICON_VIEW (appearance_capplet_get_widget (data, "wp_view"));
   gtk_icon_view_set_model (data->wp_view, GTK_TREE_MODEL (data->wp_model));
 
   g_signal_connect_after (data->wp_view, "realize",
@@ -1046,7 +1282,7 @@ desktop_init (AppearanceData *data,
 
   gtk_cell_layout_clear (GTK_CELL_LAYOUT (data->wp_view));
 
-  cr = cell_renderer_wallpaper_new ();
+  cr = gtk_cell_renderer_pixbuf_new ();
   g_object_set (cr, "xpad", 5, "ypad", 5, NULL);
 
   gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (data->wp_view), cr, TRUE);
@@ -1054,12 +1290,30 @@ desktop_init (AppearanceData *data,
                                   "pixbuf", 0,
                                   NULL);
 
-  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (data->wp_model), 2,
+  cr = gtk_cell_renderer_pixbuf_new ();
+  create_button_images (data);
+  g_object_set (cr,
+                "mode", GTK_CELL_RENDERER_MODE_ACTIVATABLE,
+                "pixbuf", buttons[0],
+                NULL);
+  g_object_set_data (G_OBJECT (cr), "buttons", GINT_TO_POINTER (TRUE));
+
+  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (data->wp_view), cr, FALSE);
+  gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (data->wp_view), cr,
+                                      buttons_cell_data_func, data, NULL);
+  g_signal_connect (data->wp_view, "selection-changed",
+                    (GCallback) wp_selected_changed_cb, data);
+  g_signal_connect (data->wp_view, "button-press-event",
+                    G_CALLBACK (wp_button_press_cb), data);
+
+  data->frame = -1;
+
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (data->wp_model), 1,
                                    (GtkTreeIterCompareFunc) wp_list_sort,
                                    data, NULL);
 
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (data->wp_model),
-                                        2, GTK_SORT_ASCENDING);
+                                        1, GTK_SORT_ASCENDING);
 
   gtk_drag_dest_set (GTK_WIDGET (data->wp_view), GTK_DEST_DEFAULT_ALL, drop_types,
                      G_N_ELEMENTS (drop_types), GDK_ACTION_COPY | GDK_ACTION_MOVE);
@@ -1071,44 +1325,51 @@ desktop_init (AppearanceData *data,
   g_signal_connect (data->wp_view, "drag-data-get",
 		    (GCallback) wp_drag_get_data, data);
 
-  data->wp_style_menu = glade_xml_get_widget (data->xml, "wp_style_menu");
+  data->wp_style_menu = appearance_capplet_get_widget (data, "wp_style_menu");
 
   g_signal_connect (data->wp_style_menu, "changed",
                     (GCallback) wp_scale_type_changed, data);
 
-  data->wp_color_menu = glade_xml_get_widget (data->xml, "wp_color_menu");
+  data->wp_color_menu = appearance_capplet_get_widget (data, "wp_color_menu");
 
   g_signal_connect (data->wp_color_menu, "changed",
                     (GCallback) wp_shade_type_changed, data);
 
-  data->wp_scpicker = glade_xml_get_widget (data->xml, "wp_scpicker");
+  data->wp_scpicker = appearance_capplet_get_widget (data, "wp_scpicker");
 
   g_signal_connect (data->wp_scpicker, "color-set",
                     (GCallback) wp_scolor_changed, data);
 
-  data->wp_pcpicker = glade_xml_get_widget (data->xml, "wp_pcpicker");
+  data->wp_pcpicker = appearance_capplet_get_widget (data, "wp_pcpicker");
 
   g_signal_connect (data->wp_pcpicker, "color-set",
                     (GCallback) wp_scolor_changed, data);
 
-  add_button = glade_xml_get_widget (data->xml, "wp_add_button");
+  add_button = appearance_capplet_get_widget (data, "wp_add_button");
   gtk_button_set_image (GTK_BUTTON (add_button),
                         gtk_image_new_from_stock ("gtk-add", GTK_ICON_SIZE_BUTTON));
 
   g_signal_connect (add_button, "clicked",
                     (GCallback) wp_file_open_dialog, data);
 
-  data->wp_rem_button = glade_xml_get_widget (data->xml, "wp_rem_button");
+  data->wp_rem_button = appearance_capplet_get_widget (data, "wp_rem_button");
 
   g_signal_connect (data->wp_rem_button, "clicked",
                     (GCallback) wp_remove_wallpaper, data);
-
-  g_idle_add (wp_load_stuffs, data);
+  data->screen_monitors_handler = g_signal_connect (gtk_widget_get_screen (GTK_WIDGET (data->wp_view)),
+                                                    "monitors-changed",
+                                                    G_CALLBACK (screen_monitors_changed),
+                                                    data);
+  data->screen_size_handler = g_signal_connect (gtk_widget_get_screen (GTK_WIDGET (data->wp_view)),
+                                                    "size-changed",
+                                                    G_CALLBACK (screen_monitors_changed),
+                                                    data);
 
   g_signal_connect (data->wp_view, "selection-changed",
                     (GCallback) wp_props_wp_selected, data);
   g_signal_connect (data->wp_view, "query-tooltip",
                     (GCallback) wp_view_tooltip_cb, data);
+  gtk_widget_set_has_tooltip (GTK_WIDGET (data->wp_view), TRUE);
 
   wp_set_sensitivities (data);
 
@@ -1121,6 +1382,18 @@ void
 desktop_shutdown (AppearanceData *data)
 {
   gnome_wp_xml_save_list (data);
+
+  if (data->screen_monitors_handler > 0) {
+    g_signal_handler_disconnect (gtk_widget_get_screen (GTK_WIDGET (data->wp_view)),
+                                 data->screen_monitors_handler);
+    data->screen_monitors_handler = 0;
+  }
+  if (data->screen_size_handler > 0) {
+    g_signal_handler_disconnect (gtk_widget_get_screen (GTK_WIDGET (data->wp_view)),
+                                 data->screen_size_handler);
+    data->screen_size_handler = 0;
+  }
+
   g_slist_foreach (data->wp_uris, (GFunc) g_free, NULL);
   g_slist_free (data->wp_uris);
   if (data->wp_filesel)
