@@ -24,6 +24,7 @@
 #endif
 
 #include <string.h>
+#include <unistd.h>
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <fcntl.h>
@@ -156,6 +157,7 @@ gtkrc_get_details (gchar *filename, GSList **engines, GSList **symbolic_colors)
 				}
 
 			}
+			close (file);
 		}
 	}
 
@@ -210,15 +212,18 @@ gtkrc_get_color_scheme (const gchar *gtkrc_file)
 			{
 				if (GINT_TO_POINTER (token) == COLOR_SCHEME_SYMBOL)
 				{
-					if (g_scanner_get_next_token (scanner) != '=')
-						continue;
-					token = g_scanner_get_next_token (scanner);
-					if (token != G_TOKEN_STRING)
-						continue;
-					g_free (result);
-					result = g_strdup (scanner->value.v_string);
+					if (g_scanner_get_next_token (scanner) == '=')
+					{
+						token = g_scanner_get_next_token (scanner);
+						if (token == G_TOKEN_STRING)
+						{
+							g_free (result);
+							result = g_strdup (scanner->value.v_string);
+						}
+					}
 				}
 			}
+			close (file);
 		}
 	}
 
